rm(list = ls())
load("/home/hrn/Dropbox/itNit/Final.rData")

library("timeSeries")
library("quantmod")
library("Hmisc")
library("forecast")
library("fMarkovSwitching")
library("tsDyn")
library("multicore")
library("doMC")
getDoParName()
options(cores=15)
registerDoMC()
getDoParWorkers()

setwd("/home/hrn/Dropbox/itNit")

source("forecastSAS.r")
source("functions.r")


fwindow=40;
ray<-cbind(rep(1:dim(sinf)[2],each=fwindow),rep.int(1:fwindow,dim(sinf)[2]));

aicAR<-aicARMA_AR<-aicARMA_MA<-bicARMA_MA<-bicAR<-bicARMA_AR<-matrix(NA,ncol=40,nrow=14)


###### AR forecasts ##########

# AR forecast are stored in aAR matrix (40 x 14) consisting of vectors, whose dimensions range between 1 and 40.

aAR<-    matrix(mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat= it.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));
aAR.nit<-matrix(mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));
bAR<-    matrix (mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat= it.sinf,  max.q=0, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));
bAR.nit<-matrix(mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf,  max.q=0, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));
dimnames(aAR)<-dimnames(aAR.nit)<-dimnames(bAR)<-dimnames(bAR.nit)<-list(1:fwindow,include.countries)
############## end of AR forecasts.##


###### ARMA forecasts ##########
# aARMA<-matrix(mapply( frcst_ARMA ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=sinf,ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
# parallel versions of ARMA estimation that naturally takes long time finding the best AIC order
aARMA.nit<-mclapply(lnit.sinf, frcst_ARMA_P_Call)
aARMA.nit<-sapply(aARMA.nit,matrix)
aARMA<-mclapply(lit.sinf, frcst_ARMA_P_Call)
aARMA<-sapply(aARMA,matrix)
############## end of ARMA forecasts.##


###### LSTAR forecasts ##########
# parallel versions of LSTAR which takes time in search of AIC order
aLSTAR.nit<-mclapply(lnit.sinf, frcst_LSTAR_P_Call)
aLSTAR.nit<-sapply(aLSTAR.nit,matrix)
aLSTAR    <-mclapply( lit.sinf, frcst_LSTAR_P_Call)
aLSTAR    <-sapply(aLSTAR,matrix)

############## end of LSTAR forecasts.##

window(nit.sinf[,13], end=c(1991,12))<-NA
###### SETAR forecasts ##########
aSETAR    <-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat= it.sinf,ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bSETAR    <-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat= it.sinf,ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));
aSETAR.nit<-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf,ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bSETAR.nit<-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf,ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));
############## end of SETAR forecasts.##


###### Neural network forecasts ##########
aNNETTS.nit<-matrix(mapply( frcst_nnetts ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf, size=3, ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
#bNNETTS.nit<-matrix(mapply( frcst_nnetts ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf, size=3, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));
aNNETTS    <-matrix(mapply( frcst_nnetts ,i=ray[,1], j=ray[,2], MoreArgs=list(dat= it.sinf, size=3, ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
#bNNETTS    <-matrix(mapply( frcst_nnetts ,i=ray[,1], j=ray[,2], MoreArgs=list(dat= it.sinf, size=3, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));
############## end of Neural Network forecasts.##


###### Random walk forecasts ##########
MSE_RW    <- matrix(mapply( function (i,j) {mean((diff(tail(na.contiguous(  it.sinf[,i]),fwindow+1 ),j))^2) } ,i=ray[,1], j=ray[,2] ),fwindow)
MSE_RW.nit<- matrix(mapply( function (i,j) {mean((diff(tail(na.contiguous( nit.sinf[,i]),fwindow+1 ),j))^2) } ,i=ray[,1], j=ray[,2] ),fwindow)
############## end of Random Walk forecasts.##
2



###### Markov regime switching forecasts ##########
MSmodels    <- multicore::mclapply( lit.sinf, function(x) MS_AR_Fit(head(x,-fwindow)))
MSmodels.nit<- multicore::mclapply(lnit.sinf, function(x) MS_AR_Fit(head(x,-fwindow))) # 7 no'da hata cikiyor.
aMS     <-matrix(mapply( function(i,j) MS_AR_For(MSmodels[[i]], head(lit.sinf[[i]],-fwindow+j-1 ), fwindow-j+1) , i=ray[,1], j=ray[,2]), fwindow)
aMS.nit <-matrix(mapply( function(i,j) MS_AR_For(MSmodels.nit[[i]], head(lnit.sinf[[i]],-fwindow+j-1 ), fwindow-j+1) , i=ray[,1], j=ray[,2]), fwindow)
############## end of Markov regime switching forecasts.##



#### Combination of forecasts ##############
combList     <- list(aAR, aARMA,aSETAR, aLSTAR, aMS, aNNETTS)
combList.nit <- list(aAR.nit, aARMA.nit,aSETAR.nit, aLSTAR.nit, aMS.nit, aNNETTS.nit)

combMat<- apply(aAR, c(1,2), function (x) (1/length(combList))* x[[1]] )

for (k in(2: length(combList))) {
  temp<- combList[[k]]
  for (j in c(1:ncol(temp) )) {
     for (i in c(1:nrow(temp) )) {
      combMat[i,j][[1]] =+ ( 1/length(combList) ) * temp[i,j][[1]]
   } 
  }
  
}
     ## now, the NIT forecasts ###
combMat.nit<- apply(aAR.nit, c(1,2), function (x) (1/length(combList.nit))* x[[1]] )

for (k in(2: length(combList.nit))) {
  temp<- combList.nit[[k]]
  for (j in c(1:ncol(temp) )) {
     for (i in c(1:nrow(temp) )) {
      combMat[i,j][[1]] =+ ( 1/length(combList.nit) ) * temp[i,j][[1]]
   } 
  }
}

############# end of Combination of forecasts ##############



series<-na.contiguous(lnit.sinf[[10]])
MS_AR_Fit( series )


# AR icin orderlari topluyoruz.
bARMA<-matrix(mapply( frcst_ARMA ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=sinf, max.q=12, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));


#dimnames(aAR)<-dimnames(aARMA)<-dimnames(bARMA)<-list(1:fwindow,include.countries)
#save.image(file="/home/ege/Documents/ARMAres.rdata")



# IT period

aAR<-matrix(mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));
  dimnames(aAR)<-list(1:fwindow,include.countries)
aARMA<-matrix(mapply( frcst_ARMA ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf,ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bAR<-matrix(mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf,  max.q=0, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));
bicAR<-bicARMA_AR # AR icin orderlari topluyoruz.
bARMA<-matrix(mapply( frcst_ARMA ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, max.q=12, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));


dimnames(aAR)<-dimnames(aARMA)<-dimnames(bARMA)<-list(1:40,include.countries)
#save.image(file="/home/ege/Documents/ARMAresIT.rdata")

# nonIT periyot

#load(file="/home/ege/Documents/ARMAresIT.rdata");
aAR<-matrix(mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));
aAR.nit<-matrix(mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));

dimnames(aAR)<-list(1:fwindow,include.countries)

aARMA<-matrix(mapply( frcst_ARMA ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf,ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));

bAR<-matrix(mapply( frcst_ARMA ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf,  max.q=0, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));

bicAR<-bicARMA_AR # AR icin orderlari topluyoruz.

bARMA<-matrix(mapply( frcst_ARMA ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf, max.q=12, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));


dimnames(aAR)<-dimnames(aARMA)<-dimnames(bARMA)<-list(1:40,include.countries)
save.image(file="/home/ege/Desktop/itNit/Final.rData")



library("multicore")
library("doMC")
getDoParName()
options(cores=16)
registerDoMC()
getDoParWorkers()


library("tsDyn")
library("quantmod")

# load(file="/home/ege/Documents/ARMAresNIT.rdata")

aAR<-matrix(mapply( frcst_AR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));
  dimnames(aAR)<-list(1:fwindow,include.countries)


aSETAR.nit<-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf, ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bSETAR.nit<-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf,ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));



aLSTAR.nit<-matrix(mapply( frcst_LSTAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));


# load(file="/home/ege/Documents/ARMAresIT.rdata")
aSETAR2<-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bSETAR<-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf,ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));

aNNETTS<-matrix(mapply( frcst_nnetts ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, size=3, ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bNNETTS<-matrix(mapply( frcst_nnetts ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, size=3, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));

# aLSTAR<-matrix(mapply( frcst_LSTAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));

realSample<-matrix(mapply( frcst_real ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));


xtable::xtable(get_MSE(aAR), caption="aAR IT", display=rep("e",15))
xtable::xtable(get_MSE(bAR), caption="bAR IT", display=rep("e",15))

xtable::xtable(get_MSE(aARMA), caption="aARMA IT", display=rep("e",15))
xtable::xtable(get_MSE(bARMA), caption="bARMA IT", display=rep("e",15))

xtable::xtable(get_MSE(aSETAR), caption="aSETAR IT", display=rep("e",15))
xtable::xtable(get_MSE(bSETAR), caption="bSETAR IT", display=rep("e",15))

xtable::xtable(get_MSE(aNNETTS), caption="aNNETTS IT", display=rep("e",15))
xtable::xtable(get_MSE(bNNETTS), caption="bNNETTS IT", display=rep("e",15))


save.image(file="/home/ege/Documents/ARMAresNIT.rdata")

load(file="/home/ege/Documents/ARMAresIT.rdata");
aSETAR<-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bSETAR<-matrix(mapply( frcst_SETAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf,ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));

aLSTAR<-matrix(mapply( frcst_LSTAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bLSTAR<-matrix(mapply( frcst_LSTAR ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));

fason<-bLSTAR
aNNETTS<-matrix(mapply( frcst_nnetts ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, size=3, ic="aic")), c(fwindow,fwindow,dim(sinf)[2]));
bNNETTS<-matrix(mapply( frcst_nnetts ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, size=3, ic="bic")), c(fwindow,fwindow,dim(sinf)[2]));

realSample<-matrix(mapply( frcst_real ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=it.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));
realSample.nit<-matrix(mapply( frcst_real ,i=ray[,1], j=ray[,2], MoreArgs=list(dat=nit.sinf, order.max=12)), c(fwindow,fwindow,dim(sinf)[2]));




xtable::xtable(get_MSE(aAR), caption="aAR IT", display=rep("e",15))
xtable::xtable(get_MSE(bAR), caption="bAR IT", display=rep("e",15))

xtable::xtable(get_MSE(aARMA), caption="aARMA IT", display=rep("e",15))
xtable::xtable(get_MSE(bARMA), caption="bARMA IT", display=rep("e",15))

xtable::xtable(get_MSE(aSETAR), caption="aSETAR IT", display=rep("e",15))
xtable::xtable(get_MSE(bSETAR), caption="bSETAR IT", display=rep("e",15))

xtable::xtable(get_MSE(aNNETTS), caption="aNNETTS IT", display=rep("e",15))
xtable::xtable(get_MSE(bNNETTS), caption="bNNETTS IT", display=rep("e",15))


save.image(file="/home/ege/Documents/ARMAresIT.rdata")
